@@ -1,59 +1,56 @@
 """Functions to manage the installation of dependencies."""
 
 import datetime
-import functools
-import os
 
 import log
+from startfile import startfile
 
-from . import common, system
-from .models import Config, Source, load_config
+from . import common
+from .decorators import preserve_cwd
+from .models import Config, Source, find_nested_configs, load_config
 
 
-def restore_cwd(func):
-    @functools.wraps(func)
-    def wrapped(*args, **kwargs):
-        cwd = os.getcwd()
-        result = func(*args, **kwargs)
-        os.chdir(cwd)
-        return result
+def init(*, force: bool = False):
+    """Create a new config file for the project.
 
-    return wrapped
+    Optional arguments:
 
+    - `force`: indicates config file should always be generated
 
-def init():
-    """Create a new config file for the project."""
+    """
     success = False
 
     config = load_config()
 
-    if config:
+    if config and not force:
         msg = "Configuration file already exists: {}".format(config.path)
-        common.show(msg, color='error')
+        common.show(msg, color="error")
 
     else:
         config = Config()
         source = Source(
-            type='git',
-            name="sample_dependency",
             repo="https://github.com/githubtraining/hellogitworld",
+            name="sample_dependency",
+            rev="master",
         )
         config.sources.append(source)
-        source = source.lock(rev="ebbbf773431ba07510251bb03f9525c7bab2b13a")
+        source = source.lock(  # type: ignore
+            rev="ebbbf773431ba07510251bb03f9525c7bab2b13a", verify_rev=False
+        )
         config.sources_locked.append(source)
         config.datafile.save()
 
         msg = "Created sample config file: {}".format(config.path)
-        common.show(msg, color='success')
+        common.show(msg, color="success")
         success = True
 
     msg = "To edit this config file, run: gitman edit"
-    common.show(msg, color='message')
+    common.show(msg, color="message")
 
     return success
 
 
-@restore_cwd
+@preserve_cwd
 def install(
     *names,
     root=None,
@@ -63,6 +60,7 @@ def install(
     force_interactive=False,
     clean=True,
     skip_changes=False,
+    skip_default_group=False,
 ):
     """Install dependencies for a project.
 
@@ -79,21 +77,28 @@ def install(
     - `clean`: indicates untracked files should be deleted from dependencies
     - `skip_changes`: indicates dependencies with uncommitted changes
      should be skipped
+    - `skip_default_group`: indicates default_group should be skipped if
+     `*names` is empty
     """
     log.info(
         "%sInstalling dependencies: %s",
-        'force-' if force or force_interactive else '',
-        ', '.join(names) if names else '<all>',
+        "force-" if force or force_interactive else "",
+        ", ".join(names) if names else "<all>",
     )
     count = None
 
     config = load_config(root)
+    configs = [config] if config else []
+    configs.extend(find_nested_configs(root, depth, []))
+    if configs:
+        count = 0
+        common.newline()
 
-    if config:
+    for config in configs:
+        common.show("Installing dependencies...", color="message", log=False)
         common.newline()
-        common.show("Installing dependencies...", color='message', log=False)
-        common.newline()
-        count = config.install_dependencies(
+
+        _count = config.install_dependencies(
             *names,
             update=False,
             depth=depth,
@@ -102,9 +107,11 @@ def install(
             fetch=fetch,
             clean=clean,
             skip_changes=skip_changes,
+            skip_default_group=skip_default_group,
         )
+        count += _count  # type: ignore
 
-        if count:
+        if _count:
             _run_scripts(
                 *names, depth=depth, force=force, _config=config, show_shell_stdout=True
             )
@@ -112,7 +119,7 @@ def install(
     return _display_result("install", "Installed", count)
 
 
-@restore_cwd
+@preserve_cwd
 def update(
     *names,
     root=None,
@@ -123,6 +130,7 @@ def update(
     clean=True,
     lock=None,  # pylint: disable=redefined-outer-name
     skip_changes=False,
+    skip_default_group=False,
 ):
     """Update dependencies for a project.
 
@@ -140,22 +148,28 @@ def update(
     - `lock`: indicates updated dependency versions should be recorded
     - `skip_changes`: indicates dependencies with uncommitted changes
      should be skipped
+    - `skip_default_group`: indicates default_group should be skipped if
+     `*names` is empty
     """
     log.info(
         "%s dependencies%s: %s",
-        'Force updating' if force or force_interactive else 'Updating',
-        ', recursively' if recurse else '',
-        ', '.join(names) if names else '<all>',
+        "Force updating" if force or force_interactive else "Updating",
+        ", recursively" if recurse else "",
+        ", ".join(names) if names else "<all>",
     )
     count = None
 
     config = load_config(root)
+    configs = [config] if config else []
+    configs.extend(find_nested_configs(root, depth, []))
+    if configs:
+        count = 0
+        common.newline()
 
-    if config:
+    for config in configs:
+        common.show("Updating dependencies...", color="message", log=False)
         common.newline()
-        common.show("Updating dependencies...", color='message', log=False)
-        common.newline()
-        count = config.install_dependencies(
+        _count = config.install_dependencies(
             *names,
             update=True,
             depth=depth,
@@ -165,10 +179,12 @@ def update(
             fetch=True,
             clean=clean,
             skip_changes=skip_changes,
+            skip_default_group=skip_default_group,
         )
+        count += _count  # type: ignore
 
-        if count and lock is not False:
-            common.show("Recording installed versions...", color='message', log=False)
+        if _count and lock is not False:
+            common.show("Recording installed versions...", color="message", log=False)
             common.newline()
             config.lock_dependencies(
                 *names,
@@ -176,7 +192,7 @@ def update(
                 skip_changes=skip_changes or force_interactive,
             )
 
-        if count:
+        if _count:
             _run_scripts(
                 *names, depth=depth, force=force, _config=config, show_shell_stdout=True
             )
@@ -199,14 +215,14 @@ def _run_scripts(
     """
     assert _config, "'_config' is required"
 
-    common.show("Running scripts...", color='message', log=False)
+    common.show("Running scripts...", color="message", log=False)
     common.newline()
     _config.run_scripts(
         *names, depth=depth, force=force, show_shell_stdout=show_shell_stdout
     )
 
 
-@restore_cwd
+@preserve_cwd
 def display(*, root=None, depth=None, allow_dirty=True):
     """Display installed dependencies for a project.
 
@@ -219,32 +235,49 @@ def display(*, root=None, depth=None, allow_dirty=True):
     """
     log.info("Displaying dependencies...")
     count = None
-
     config = load_config(root)
 
-    if config:
+    if config and config.root:
         common.newline()
         common.show(
-            "Displaying current dependency versions...", color='message', log=False
+            "Displaying current dependency versions...", color="message", log=False
         )
         common.newline()
         config.log(datetime.datetime.now().strftime("%F %T"))
         count = 0
+
+        skip_paths = []
         for identity in config.get_dependencies(depth=depth, allow_dirty=allow_dirty):
             count += 1
             config.log("{}: {} @ {}", *identity)
+            skip_paths.append(identity.path)
+
+        nested_configs = find_nested_configs(config.root, depth, skip_paths)
+        if nested_configs:
+            common.show(
+                "Displaying nested dependency versions...", color="message", log=False
+            )
+            common.newline()
+            for nested_config in nested_configs:
+                for identity in nested_config.get_dependencies(
+                    depth=depth, allow_dirty=allow_dirty
+                ):
+                    count += 1
+                    config.log("{}: {} @ {}", *identity)
+
         config.log()
 
     return _display_result("display", "Displayed", count)
 
 
-@restore_cwd
-def lock(*names, root=None):
+@preserve_cwd
+def lock(*names, depth=None, root=None):
     """Lock current dependency versions for a project.
 
     Optional arguments:
 
     - `*names`: optional list of dependency directory names to filter on
+    - `depth`: number of levels of dependencies to traverse
     - `root`: specifies the path to the root working tree
 
     """
@@ -252,18 +285,22 @@ def lock(*names, root=None):
     count = None
 
     config = load_config(root)
+    configs = [config] if config else []
+    configs.extend(find_nested_configs(root, depth, []))
+    if configs:
+        count = 0
+        common.newline()
 
-    if config:
+    for config in configs:
+        common.show("Locking dependencies...", color="message", log=False)
         common.newline()
-        common.show("Locking dependencies...", color='message', log=False)
-        common.newline()
-        count = config.lock_dependencies(*names, obey_existing=False)
+        count += config.lock_dependencies(*names, obey_existing=False)  # type: ignore
         common.dedent(level=0)
 
     return _display_result("lock", "Locked", count)
 
 
-@restore_cwd
+@preserve_cwd
 def delete(*, root=None, force=False, keep_location=False):
     """Delete dependencies for a project.
 
@@ -276,18 +313,17 @@ def delete(*, root=None, force=False, keep_location=False):
     """
     log.info("Deleting dependencies...")
     count = None
-
     config = load_config(root)
 
     if config:
         common.newline()
-        common.show("Checking for uncommitted changes...", color='message', log=False)
+        common.show("Checking for uncommitted changes...", color="message", log=False)
         common.newline()
         count = len(list(config.get_dependencies(allow_dirty=force)))
         common.dedent(level=0)
-        common.show("Deleting all dependencies...", color='message', log=False)
+        common.show("Deleting all dependencies...", color="message", log=False)
         common.newline()
-        if keep_location:
+        if keep_location or config.location == ".":
             config.clean_dependencies()
         else:
             config.uninstall_dependencies()
@@ -303,7 +339,6 @@ def show(*names, root=None):
 
     """
     log.info("Finding paths...")
-
     config = load_config(root)
 
     if not config:
@@ -311,13 +346,13 @@ def show(*names, root=None):
         return False
 
     for name in names or [None]:
-        common.show(config.get_path(name), color='path')
+        common.show(config.get_path(name), color="path")
 
     return True
 
 
 def edit(*, root=None):
-    """Open the confuration file for a project.
+    """Open the configuration file for a project.
 
     Optional arguments:
 
@@ -325,17 +360,16 @@ def edit(*, root=None):
 
     """
     log.info("Launching config...")
-
     config = load_config(root)
 
     if not config:
         log.error("No config found")
         return False
 
-    return system.launch(config.path)
+    return startfile(config.path)
 
 
-def _display_result(present, past, count, allow_zero=False):
+def _display_result(modify, modified, count, allow_zero=False):
     """Convert a command's dependency count to a return status.
 
     >>> _display_result("sample", "Sampled", 1)
@@ -352,11 +386,12 @@ def _display_result(present, past, count, allow_zero=False):
 
     """
     if count is None:
-        log.warning("No dependencies to %s", present)
+        common.show(f"No dependencies to {modify}", color="error")
     elif count == 1:
-        log.info("%s 1 dependency", past)
+        common.show(f"{modified} 1 dependency", color="message")
     else:
-        log.info("%s %s dependencies", past, count)
+        common.show(f"{modified} {count} dependencies", color="message")
+    common.newline()
 
     if count:
         return True
