@@ -1,14 +1,14 @@
 import os
 import sys
-from typing import List, Optional
+from typing import Iterator, List, Optional
 
 import log
 from datafiles import datafile, field
 
-from .. import common, exceptions, shell
+from .. import common, exceptions, settings, shell
 from ..decorators import preserve_cwd
 from .group import Group
-from .source import Source
+from .source import Identity, Source
 
 
 @datafile("{self.root}/{self.filename}", defaults=True, manual=True)
@@ -54,16 +54,16 @@ class Config:
                 if source.name == group.name:
                     msg = (
                         "Name conflict detected between source name and "
-                        "group name \"{}\""
+                        'group name "{}"'
                     ).format(source.name)
                     raise exceptions.InvalidConfig(msg)
 
-    def get_path(self, name=None):
+    def get_path(self, name: Optional[str] = None) -> str:
         """Get the full path to a dependency or internal file."""
         base = self.location_path
-        if name == '__config__':
-            return self.path
-        if name == '__log__':
+        if name == "__config__":
+            return self.path  # type: ignore
+        if name == "__log__":
             return self.log_path
         if name:
             return os.path.normpath(os.path.join(base, name))
@@ -71,17 +71,17 @@ class Config:
 
     def install_dependencies(
         self,
-        *names,
-        depth=None,
-        update=True,
-        recurse=False,
-        force=False,
-        force_interactive=False,
-        fetch=False,
-        clean=True,
-        skip_changes=False,
-        skip_default_group=False,
-    ):  # pylint: disable=too-many-locals
+        *names: str,
+        depth: Optional[int] = None,
+        update: bool = True,
+        recurse: bool = False,
+        force: bool = False,
+        force_interactive: bool = False,
+        fetch: bool = False,
+        clean: bool = True,
+        skip_changes: bool = False,
+        skip_default_group: bool = False,
+    ) -> int:
         """Download or update the specified dependencies."""
         if depth == 0:
             log.info("Skipped directory: %s", self.location_path)
@@ -113,7 +113,7 @@ class Config:
                 clean=clean,
                 skip_changes=skip_changes,
             )
-            source.create_link(self.root, force=force)
+            assert self.root, f"Missing root: {self}"
             source.create_links(self.root, force=force)
             common.newline()
             count += 1
@@ -136,14 +136,21 @@ class Config:
             shell.cd(self.location_path, _show=False)
 
         common.dedent()
+
         if sources_filter:
-            log.error("No such dependency: %s", ' '.join(sources_filter))
+            log.error("No such dependency: %s", " ".join(sources_filter))
             return 0
 
         return count
 
     @preserve_cwd
-    def run_scripts(self, *names, depth=None, force=False, show_shell_stdout=False):
+    def run_scripts(
+        self,
+        *names: str,
+        depth: Optional[int] = None,
+        force: bool = False,
+        show_shell_stdout: bool = False,
+    ) -> int:
         """Run scripts for the specified dependencies."""
         if depth == 0:
             log.info("Skipped directory: %s", self.location_path)
@@ -181,11 +188,39 @@ class Config:
 
         return count
 
-    def lock_dependencies(self, *names, obey_existing=True, skip_changes=False):
+    @classmethod
+    def _split_name_and_rev(cls, name_rev):
+        true_name = name_rev
+        rev = None
+        if "@" in name_rev:
+            name_split = name_rev.split("@")
+            true_name = name_split[0]
+            rev = name_split[1]
+        return true_name, rev
+
+    @classmethod
+    def _remap_names_and_revs(cls, names):
+        name_rev_map = {}
+        for name in names:
+            base_name, rev = cls._split_name_and_rev(name)
+            name_rev_map[base_name] = rev
+        return name_rev_map.keys(), name_rev_map
+
+    def lock_dependencies(
+        self, *names: str, obey_existing: bool = True, skip_changes: bool = False
+    ) -> int:
         """Lock down the immediate dependency versions."""
+        sources_to_install, source_to_install_revs = self._remap_names_and_revs(
+            [*names]
+        )
         sources = self._get_sources(use_locked=obey_existing).copy()
+
+        skip_default = True
+        if len(sources_to_install) == 0:
+            skip_default = False
+
         sources_filter = self._get_sources_filter(
-            *names, sources=sources, skip_default_group=False
+            *sources_to_install, sources=sources, skip_default_group=skip_default
         )
 
         shell.cd(self.location_path)
@@ -198,7 +233,10 @@ class Config:
                 log.info("Skipped dependency: %s", source.name)
                 continue
 
-            source_locked = source.lock(skip_changes=skip_changes)
+            rev = None
+            if source.name in source_to_install_revs.keys():
+                rev = source_to_install_revs[source.name]
+            source_locked = source.lock(skip_changes=skip_changes, rev=rev)
 
             if source_locked is not None:
                 try:
@@ -255,7 +293,9 @@ class Config:
 
         common.dedent()
 
-    def get_dependencies(self, depth=None, allow_dirty=True):
+    def get_dependencies(
+        self, depth: Optional[int] = None, allow_dirty: bool = True
+    ) -> Iterator[Identity]:
         """Yield the path, repository, and hash of each dependency."""
         if not os.path.exists(self.location_path):
             return
@@ -264,7 +304,7 @@ class Config:
         common.newline()
         common.indent()
 
-        for source in self.sources:
+        for source in self._get_sources(use_locked=False):
 
             if depth == 0:
                 log.info("Skipped dependency: %s", source.name)
@@ -285,18 +325,18 @@ class Config:
 
         common.dedent()
 
-    def log(self, message="", *args):
+    def log(self, message: str = "", *args):
         """Append a message to the log file."""
         os.makedirs(self.location_path, exist_ok=True)
-        with open(self.log_path, 'a') as outfile:
-            outfile.write(message.format(*args) + '\n')
+        with open(self.log_path, "a") as outfile:
+            outfile.write(message.format(*args) + "\n")
 
-    def _get_sources(self, *, use_locked=None):
+    def _get_sources(self, *, use_locked: Optional[bool] = None) -> List[Source]:
         """Merge source lists using the requested section as the base."""
         if use_locked is True:
             if self.sources_locked:
                 return self.sources_locked
-            log.info("No locked sources, defaulting to none...")
+            log.info("No locked sources, defaulting to none")
             return []
 
         sources: List[Source] = []
@@ -304,10 +344,10 @@ class Config:
             sources = self.sources
         else:
             if self.sources_locked:
-                log.info("Defaulting to locked sources...")
+                log.info("Defaulting to locked sources")
                 sources = self.sources_locked
             else:
-                log.info("No locked sources, using latest...")
+                log.info("No locked sources, using latest")
                 sources = self.sources
 
         extras = []
@@ -318,8 +358,10 @@ class Config:
 
         return sources + extras
 
-    def _get_sources_filter(self, *names, sources, skip_default_group):
-        """Get filtered sublist of sources."""
+    def _get_sources_filter(
+        self, *names: str, sources: List[Source], skip_default_group: bool
+    ) -> List[str]:
+        """Get a filtered subset of sources."""
         names_list = list(names)
 
         if not names_list and not skip_default_group:
@@ -335,12 +377,14 @@ class Config:
         )
 
         if not sources_filter:
-            sources_filter = [source.name for source in sources]
+            sources_filter = [source.name for source in sources if source.name]
 
         return list(set(sources_filter))
 
 
-def load_config(start=None, *, search=True):
+def load_config(
+    start: Optional[str] = None, *, search: bool = True
+) -> Optional[Config]:
     """Load the config for the current project."""
     start = os.path.abspath(start) if start else _resolve_current_directory()
 
@@ -371,6 +415,37 @@ def load_config(start=None, *, search=True):
     return None
 
 
+def find_nested_configs(
+    root: str, depth: Optional[int], skip_paths: List[str]
+) -> List[Config]:
+    """Find all other projects in the same directory."""
+    root = os.path.abspath(root) if root else _resolve_current_directory()
+    configs: List[Config] = []
+
+    if (depth is not None and depth <= 1) or settings.CI:
+        return configs
+
+    log.debug(f"Searching for nested project in: {root}")
+    for name in os.listdir(root):
+        if name.startswith("."):
+            continue
+        path = os.path.join(root, name)
+        if os.path.isdir(path) and path not in skip_paths:
+            config = load_config(path, search=False)
+            if config:
+                configs.append(config)
+
+            if os.path.islink(path):
+                continue
+
+            if depth is not None:
+                configs.extend(find_nested_configs(path, depth - 1, skip_paths))
+            else:
+                configs.extend(find_nested_configs(path, depth, skip_paths))
+
+    return configs
+
+
 def _resolve_current_directory():
     start = os.getcwd()
     if sys.version_info < (3, 8) and os.name == "nt":
@@ -383,6 +458,6 @@ def _resolve_current_directory():
 
 def _valid_filename(filename):
     name, ext = os.path.splitext(filename.lower())
-    if name.startswith('.'):
+    if name.startswith("."):
         name = name[1:]
-    return name in {'gitman', 'gdm'} and ext in {'.yml', '.yaml'}
+    return name in {"gitman", "gdm"} and ext in {".yml", ".yaml"}
